@@ -14,7 +14,6 @@
 
 // Qt includes
 
-#include <QTextCodec>
 #include <QBuffer>
 
 // Local includes
@@ -29,7 +28,12 @@ bool KExiv2::canWriteExif(const QString& filePath)
 {
     try
     {
-        Exiv2::Image::AutoPtr image = Exiv2::ImageFactory::open((const char*)
+#if EXIV2_TEST_VERSION(0,28,0)
+        Exiv2::Image::UniquePtr image =
+#else
+        Exiv2::Image::AutoPtr image =
+#endif
+                                      Exiv2::ImageFactory::open((const char*)
                                       (QFile::encodeName(filePath).constData()));
 
         Exiv2::AccessMode mode = image->checkMode(Exiv2::mdExif);
@@ -40,7 +44,12 @@ bool KExiv2::canWriteExif(const QString& filePath)
     {
         std::string s(e.what());
         qCCritical(LIBKEXIV2_LOG) << "Cannot check Exif access mode using Exiv2 (Error #"
-                    << e.code() << ": " << s.c_str() << ")";
+#if EXIV2_TEST_VERSION(0,28,0)
+                    << Exiv2::Error(e.code()).what()
+#else
+                    << e.code() << ": " << s.c_str()
+#endif
+                    << ")";
     }
     catch(...)
     {
@@ -266,19 +275,9 @@ QString KExiv2::getExifComment() const
     return QString();
 }
 
-static bool is7BitAscii(const QByteArray& s)
+static bool is7BitAscii(const QString& s)
 {
-    const int size = s.size();
-
-    for (int i=0; i<size; i++)
-    {
-        if (!isascii(s[i]))
-        {
-            return false;
-        }
-    }
-
-    return true;
+    return std::all_of(s.begin(), s.end(), [](QChar c) { return c.unicode() <= 0x7f; });
 }
 
 bool KExiv2::setExifComment(const QString& comment, bool setProgramName) const
@@ -296,19 +295,13 @@ bool KExiv2::setExifComment(const QString& comment, bool setProgramName) const
             setExifTagString("Exif.Image.ImageDescription", comment, setProgramName);
 
             // Write as Unicode only when necessary.
-            QTextCodec* latin1Codec = QTextCodec::codecForName("iso8859-1");
-            if (latin1Codec->canEncode(comment))
+            if (is7BitAscii(comment))
             {
-                // We know it's in the ISO-8859-1 8bit range.
-                // Check if it's in the ASCII 7bit range
-                if (is7BitAscii(comment.toLatin1()))
-                {
-                    // write as ASCII
-                    std::string exifComment("charset=\"Ascii\" ");
-                    exifComment += comment.toLatin1().constData();
-                    d->exifMetadata()["Exif.Photo.UserComment"] = exifComment;
-                    return true;
-                }
+                // write as ASCII
+                std::string exifComment("charset=\"Ascii\" ");
+                exifComment += comment.toLatin1().constData();
+                d->exifMetadata()["Exif.Photo.UserComment"] = exifComment;
+                return true;
             }
             // write as Unicode (UCS-2)
             std::string exifComment("charset=\"Unicode\" ");
@@ -501,16 +494,20 @@ bool KExiv2::setExifTagData(const char* exifTagName, const QByteArray& data, boo
 bool KExiv2::setExifTagVariant(const char* exifTagName, const QVariant& val,
                                bool rationalWantSmallDenominator, bool setProgramName) const
 {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    switch (val.metaType().id())
+#else
     switch (val.type())
+#endif
     {
-        case QVariant::Int:
-        case QVariant::UInt:
-        case QVariant::Bool:
-        case QVariant::LongLong:
-        case QVariant::ULongLong:
+        case QMetaType::Int:
+        case QMetaType::UInt:
+        case QMetaType::Bool:
+        case QMetaType::LongLong:
+        case QMetaType::ULongLong:
             return setExifTagLong(exifTagName, val.toInt(), setProgramName);
 
-        case QVariant::Double:
+        case QMetaType::Double:
         {
             long num, den;
 
@@ -521,7 +518,7 @@ bool KExiv2::setExifTagVariant(const char* exifTagName, const QVariant& val,
 
             return setExifTagRational(exifTagName, num, den, setProgramName);
         }
-        case QVariant::List:
+        case QMetaType::QVariantList:
         {
             long num = 0, den = 1;
             QList<QVariant> list = val.toList();
@@ -535,8 +532,8 @@ bool KExiv2::setExifTagVariant(const char* exifTagName, const QVariant& val,
             return setExifTagRational(exifTagName, num, den, setProgramName);
         }
 
-        case QVariant::Date:
-        case QVariant::DateTime:
+        case QMetaType::QDate:
+        case QMetaType::QDateTime:
         {
             QDateTime dateTime = val.toDateTime();
 
@@ -563,11 +560,11 @@ bool KExiv2::setExifTagVariant(const char* exifTagName, const QVariant& val,
             return false;
         }
 
-        case QVariant::String:
-        case QVariant::Char:
+        case QMetaType::QString:
+        case QMetaType::QChar:
             return setExifTagString(exifTagName, val.toString(), setProgramName);
 
-        case QVariant::ByteArray:
+        case QMetaType::QByteArray:
             return setExifTagData(exifTagName, val.toByteArray(), setProgramName);
         default:
             break;
@@ -582,19 +579,23 @@ QString KExiv2::createExifUserStringFromValue(const char* exifTagName, const QVa
         Exiv2::ExifKey key(exifTagName);
         Exiv2::Exifdatum datum(key);
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+        switch (val.metaType().id())
+#else
         switch (val.type())
+#endif
         {
-            case QVariant::Int:
-            case QVariant::Bool:
-            case QVariant::LongLong:
-            case QVariant::ULongLong:
+            case QMetaType::Int:
+            case QMetaType::Bool:
+            case QMetaType::LongLong:
+            case QMetaType::ULongLong:
                 datum = (int32_t)val.toInt();
                 break;
-            case QVariant::UInt:
+            case QMetaType::UInt:
                 datum = (uint32_t)val.toUInt();
                 break;
 
-            case QVariant::Double:
+            case QMetaType::Double:
             {
                 long num, den;
                 convertToRationalSmallDenominator(val.toDouble(), &num, &den);
@@ -604,7 +605,7 @@ QString KExiv2::createExifUserStringFromValue(const char* exifTagName, const QVa
                 datum = rational;
                 break;
             }
-            case QVariant::List:
+            case QMetaType::QVariantList:
             {
                 long num = 0, den = 1;
                 QList<QVariant> list = val.toList();
@@ -619,8 +620,8 @@ QString KExiv2::createExifUserStringFromValue(const char* exifTagName, const QVa
                 break;
             }
 
-            case QVariant::Date:
-            case QVariant::DateTime:
+            case QMetaType::QDate:
+            case QMetaType::QDateTime:
             {
                 QDateTime dateTime = val.toDateTime();
                 if(!dateTime.isValid())
@@ -631,8 +632,8 @@ QString KExiv2::createExifUserStringFromValue(const char* exifTagName, const QVa
                 break;
             }
 
-            case QVariant::String:
-            case QVariant::Char:
+            case QMetaType::QString:
+            case QMetaType::QChar:
                 datum = (std::string)val.toString().toLatin1().constData();
                 break;
             default:
@@ -675,7 +676,11 @@ bool KExiv2::getExifTagLong(const char* exifTagName, long& val, int component) c
 
         if (it != exifData.end() && it->count() > 0)
         {
+#if EXIV2_TEST_VERSION(0,28,0)
+            val = it->toUint32(component);
+#else
             val = it->toLong(component);
+#endif
             return true;
         }
     }
@@ -739,16 +744,28 @@ QVariant KExiv2::getExifTagVariant(const char* exifTagName, bool rationalAsListO
                 case Exiv2::signedShort:
                 case Exiv2::signedLong:
                     if (it->count() > component)
+#if EXIV2_TEST_VERSION(0,28,0)
+                        return QVariant((int)it->toUint32(component));
+#else
                         return QVariant((int)it->toLong(component));
+#endif
                     else
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+                        return QVariant(QMetaType(QMetaType::Int));
+#else
                         return QVariant(QVariant::Int);
+#endif
                 case Exiv2::unsignedRational:
                 case Exiv2::signedRational:
 
                     if (rationalAsListOfInts)
                     {
                         if (it->count() <= component)
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+                            return QVariant(QMetaType(QMetaType::QVariantList));
+#else
                             return QVariant(QVariant::List);
+#endif
 
                         QList<QVariant> list;
                         list << (*it).toRational(component).first;
@@ -759,14 +776,22 @@ QVariant KExiv2::getExifTagVariant(const char* exifTagName, bool rationalAsListO
                     else
                     {
                         if (it->count() <= component)
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+                            return QVariant(QMetaType(QMetaType::Double));
+#else
                             return QVariant(QVariant::Double);
+#endif
 
                         // prefer double precision
                         double num = (*it).toRational(component).first;
                         double den = (*it).toRational(component).second;
 
                         if (den == 0.0)
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+                            return QVariant(QMetaType(QMetaType::Double));
+#else
                             return QVariant(QVariant::Double);
+#endif
 
                         return QVariant(num / den);
                     }
@@ -871,7 +896,11 @@ QImage KExiv2::getExifThumbnail(bool fixOrientation) const
     {
         Exiv2::ExifThumbC thumb(d->exifMetadata());
         Exiv2::DataBuf const c1 = thumb.copy();
+#if EXIV2_TEST_VERSION(0,28,0)
+        thumbnail.loadFromData(c1.c_data(), c1.size());
+#else
         thumbnail.loadFromData(c1.pData_, c1.size_);
+#endif
 
         if (!thumbnail.isNull())
         {
@@ -887,7 +916,11 @@ QImage KExiv2::getExifThumbnail(bool fixOrientation) const
 
                 if (it != exifData.end() && it->count())
                 {
+#if EXIV2_TEST_VERSION(0,28,0)
+                    long orientation = it->toUint32();
+#else
                     long orientation = it->toLong();
+#endif
                     qCDebug(LIBKEXIV2_LOG) << "Exif Thumbnail Orientation: " << (int)orientation;
                     rotateExifQImage(thumbnail, (ImageOrientation)orientation);
                 }
@@ -965,9 +998,15 @@ bool KExiv2::setTiffThumbnail(const QImage& thumbImage, bool setProgramName) con
         // Make sure IFD0 is explicitly marked as a main image
         Exiv2::ExifData::const_iterator pos = d->exifMetadata().findKey(Exiv2::ExifKey("Exif.Image.NewSubfileType"));
 
+#if EXIV2_TEST_VERSION(0,28,0)
+        if (pos == d->exifMetadata().end() || pos->count() != 1 || pos->toUint32() != 0)
+#else
         if (pos == d->exifMetadata().end() || pos->count() != 1 || pos->toLong() != 0)
+#endif
         {
-#if EXIV2_TEST_VERSION(0,27,0)
+#if EXIV2_TEST_VERSION(0,28,0)
+            throw Exiv2::Error(Exiv2::ErrorCode::kerErrorMessage, "Exif.Image.NewSubfileType missing or not set as main image");
+#elif EXIV2_TEST_VERSION(0,27,0)
             throw Exiv2::Error(Exiv2::kerErrorMessage, "Exif.Image.NewSubfileType missing or not set as main image");
 #else
             throw Exiv2::Error(1, "Exif.Image.NewSubfileType missing or not set as main image");
@@ -996,9 +1035,17 @@ bool KExiv2::setTiffThumbnail(const QImage& thumbImage, bool setProgramName) con
             Exiv2::DataBuf buf((Exiv2::byte *)data.data(), data.size());
             Exiv2::ULongValue val;
             val.read("0");
+#if EXIV2_TEST_VERSION(0,28,0)
+            val.setDataArea(buf.c_data(), buf.size());
+#else
             val.setDataArea(buf.pData_, buf.size_);
+#endif
             d->exifMetadata()["Exif.SubImage1.JPEGInterchangeFormat"]       = val;
+#if EXIV2_TEST_VERSION(0,28,0)
+            d->exifMetadata()["Exif.SubImage1.JPEGInterchangeFormatLength"] = uint32_t(buf.size());
+#else
             d->exifMetadata()["Exif.SubImage1.JPEGInterchangeFormatLength"] = uint32_t(buf.size_);
+#endif
             d->exifMetadata()["Exif.SubImage1.Compression"]                 = uint16_t(6); // JPEG (old-style)
             d->exifMetadata()["Exif.SubImage1.NewSubfileType"]              = uint32_t(1); // Thumbnail image
             return true;
